precision highp float;

layout( push_constant ) uniform PushConstants
{
	vec3 v3LightColor;
	float fMaxBias;
} p;

layout(set = 0, binding = 1001) uniform lowp sampler2D texShadowMap;

in highp vec4 v_v4ShadowMapPosition;
in highp vec2 v_v2TexCoordLightmap;
in lowp vec3 v_v3Normal;
in vec3 v_v3LightDir;

layout(location = 0) out vec4 out_v4Color;

void main() 
{
#ifndef SPOT_LIGHT
	if (v_v4ShadowMapPosition.x > 1.0)
	{
		discard;
	}
	else if (v_v4ShadowMapPosition.x < 0.0)
	{
		discard;
	}
	else if (v_v4ShadowMapPosition.y > 1.0)
	{
		discard;
	}
	else if (v_v4ShadowMapPosition.y < 0.0)
	{
		discard;
	}
	else if (v_v4ShadowMapPosition.z < 0.0)
	{
		discard;
	}
	else if (v_v4ShadowMapPosition.z > 1.0)
	{
		discard;
	}
	else
#endif
	{	

#ifdef SPOT_LIGHT
	float fBias = (0.005 * tan(acos(clamp(dot(v_v3Normal, v_v3LightDir), 0.0, 1.0))));
	fBias = clamp(fBias, 0.0, p.fMaxBias);
#else
	float fMaxBias = 0.000;
	float fBias = (fMaxBias * tan(acos(clamp(dot(v_v3Normal, v_v3LightDir), 0.0, 1.0))));
	fBias = clamp(fBias, 0.0, fMaxBias);
#endif

	float fShadowZWithBias = (v_v4ShadowMapPosition.z - fBias);
	vec2 v2ShadowPos = v_v4ShadowMapPosition.xy / v_v4ShadowMapPosition.w;

	float fVisibility = 1.0;
	float amount = 0.75*0.25;
	if ( texture ( texShadowMap, v2ShadowPos + vec2( -0.94201624,  -0.39906216 )/1000.0 ).r  <  fShadowZWithBias ) 
		fVisibility -= amount;
	if ( texture( texShadowMap, v2ShadowPos + vec2(  0.94558609,  -0.76890725 )/1000.0 ).r  <  fShadowZWithBias ) 
		fVisibility -= amount;
	if ( texture( texShadowMap, v2ShadowPos + vec2( -0.094184101, -0.92938870 )/1000.0 ).r  <  fShadowZWithBias ) 
		fVisibility -= amount;
	if ( texture( texShadowMap, v2ShadowPos + vec2(  0.34495938,   0.29387760 )/1000.0 ).r  <  fShadowZWithBias ) 
		fVisibility -= amount;

	float far = 100.0;
	float farAmount = 0.25*0.25;
	if ( texture ( texShadowMap, v2ShadowPos + vec2( -0.94201624,  -0.39906216 )/far ).r  <  fShadowZWithBias ) 
		fVisibility -= farAmount;
	if ( texture( texShadowMap, v2ShadowPos + vec2(  0.94558609,  -0.76890725 )/far ).r  <  fShadowZWithBias ) 
		fVisibility -= farAmount;
	if ( texture( texShadowMap, v2ShadowPos + vec2( -0.094184101, -0.92938870 )/far ).r  <  fShadowZWithBias ) 
		fVisibility -= farAmount;
	if ( texture( texShadowMap, v2ShadowPos + vec2(  0.34495938,   0.29387760 )/far ).r  <  fShadowZWithBias ) 
		fVisibility -= farAmount;

	float fAngleMod = dot(v_v3Normal, v_v3LightDir);
	fVisibility *= clamp(-exp(fAngleMod) + 2.0, 0.0, 1.0);

#ifdef SPOT_LIGHT
	vec2 v2ShadowDist = (v_v4ShadowMapPosition.xy * 2.0) - 1.0;
	float fI = ((2.0 - dot(v2ShadowDist, v2ShadowDist)) * 0.5) - 0.5;
	fI = clamp(fI, 0.0, 1.0);
	out_v4Color = vec4(p.v3LightColor, fVisibility * fI);	
#else
	out_v4Color = vec4(p.v3LightColor, fVisibility);	
#endif
	}
}
