precision lowp float;

layout(location = 0) out vec4 out_v4Color;

layout(binding = 1001) uniform lowp sampler2D texDiffuseMap;
layout(binding = 1002) uniform lowp sampler2D texNormalMap;
layout(binding = 1003) uniform lowp sampler2D texSpecularWearAoMap;
layout(binding = 1004) uniform lowp samplerCube texCubeMap;

in vec2 v_v2TexCoord;
in vec3 v_v3WorldPosition;

layout(set = 0, binding = 0) uniform U
{    
	vec3 v3SkinColour;
	mat3 m33NormalTransform;
	vec4 v4Color;
	vec3 v3LightDir;
	vec3 v3EyePosition;
	float fBrightness;
} u;

//varying vec2 v_v2WearUV;
//uniform vec3 u_v3WearColour;

void main() 
{	
	vec3 v3TextureNormal = texture(texNormalMap, v_v2TexCoord).rgb; 
	vec3 v3TextureDiffuse = texture(texDiffuseMap, v_v2TexCoord).rgb;
	vec3 v3TextureWearSpecAo = texture(texSpecularWearAoMap, v_v2TexCoord).rgb; // r contains specular, g contains wear, b contains ambient oclusion
	

	//v3TextureNormal.w = v4TextureWearPattern.r;
	//v3TextureDiffuse.w = v4TextureWearPattern.b;

	v3TextureDiffuse *= u.v3SkinColour;
		
	// todo: Is there a way to pre transform to avoid multiplying by u.m33NormalTransform during the fragment shader?
	// todo: I think it might require the cube map to be rotated, which wont work for different car body parts.
	// todo: It is an expensive per pixel operation so it would be nice to avoid it.
	vec3 v3Incident = v_v3WorldPosition - u.v3EyePosition;
	vec3 v3Normal = (2.0 * (v3TextureNormal.xyz - vec3(0.5, 0.5, 0.5))) * u.m33NormalTransform;
	v3Incident = reflect(v3Incident, v3Normal);	

	vec4 v4TextureCube = texture(texCubeMap, v3Incident);

	//vec4 v4TextureWear = texture2D(tex4, v_v2WearUV);		 
	//vec4 v4Wear = vec4(u.v3WearColour, (v3TextureWearSpecAo.g - 1.0) / (0.001 + v4TextureWear.r) + 1.0);
	//v4Wear.a = clamp(v4Wear.a, 0.0, 1.0);	
	//float fA = clamp((1.0 - v4Wear.a * 1.5), 0.0, 1.5);
	//v3TextureDiffuse.xyz = (v4Wear.rgb * v4Wear.a * v4Wear.a) + (v3TextureDiffuse.xyz * fA);
	//v3TextureWearSpecAo.r *= fA;


	//out_v4Color.xyz = 
	//	v4TextureCube.xyz * (v3TextureWearSpecAo.r * v3TextureWearSpecAo.b) * 0.5 + 
	//	v3TextureDiffuse.xyz * ((dot(v3Normal, u.v3LightDir) * vec3(1.2, 1.4, 1.6)) * v3TextureWearSpecAo.b + vec3(.6, .35, 0.15)) * u.v4Color.xyz;

	out_v4Color.xyz = 
		v4TextureCube.xyz * (v3TextureWearSpecAo.r * v3TextureWearSpecAo.b) * 0.5 + 
		v3TextureDiffuse.xyz * ((dot(v3Normal, u.v3LightDir) * vec3(0.8, 0.9, 1.0)) * v3TextureWearSpecAo.b + vec3(.6 + 0.6, .35 + 0.6, 0.15 + 0.6)) * u.v4Color.xyz;

	
	
	//out_v4Color.rgb = v3TextureWearSpecAo.rgb;
				
	vec3 v3Tmp = 1.0 - out_v4Color.rgb;

	out_v4Color.rgb = out_v4Color.rgb * (2.0 - u.fBrightness) + (1.0 - v3Tmp * v3Tmp * v3Tmp) * (u.fBrightness - 1.0);

//	out_v4Color.rgb = v3TextureNormal.rgb;
	
	out_v4Color.w = 0.25;
}
