
precision lowp float;

layout(location = 0) out vec4 out_v4Color;

layout(binding = 0) uniform U
{    
	mat4 m44ModelViewProjection;
	mat3 m33SpecularRotation;
	mediump vec3 v3EyePosition;
	lowp float fBrightness;
	lowp float fRewindEffect;
	float fGamma;
	float fZOffset;
	float fZClamp;
	float fMinZOffset;  
} u;

layout(set = 2, binding = 0) uniform U2
{
	float fAlphaLevel;
	lowp float fSpecular;
} u2;

layout(location = 0) in vec2 v_v2TexCoord;
layout(location = 1) in vec4 v_v4Color;

layout(set = 1, binding = 1001) uniform lowp sampler2D tex;
#ifndef DETAIL_LOW
	layout(set = 0, binding = 1002) uniform lowp samplerCube texCubeMap;
#endif

#ifdef DETAIL_MEDIUM
	layout(location = 4) in mediump vec3 v_v3Reflection;
#endif

#ifdef DETAIL_HIGH
	layout(location = 2) in lowp vec3 v_v3Normal;
	layout(location = 3) in mediump vec3 v_v3WorldPosition;
#endif

void main() 
{
	vec4 v4Texture = texture(tex, v_v2TexCoord) * v_v4Color;

	#ifdef DETAIL_HIGH
		mediump vec3 v3Incident = v_v3WorldPosition - u.v3EyePosition;
		mediump vec3 v3Reflection = v3Incident - 2.0 * dot(v3Incident, v_v3Normal) * v_v3Normal / dot(v_v3Normal, v_v3Normal);	
		lowp vec4 v4TextureCube = texture(texCubeMap, v3Reflection);

		lowp float fSpec = u2.fSpecular * ((v4Texture.r + v4Texture.g + v4Texture.b) * 0.333);

		lowp vec3 v3Out = v4Texture.rgb + v4TextureCube.rgb * u2.fSpecular * fSpec;
	#endif

	#ifdef DETAIL_MEDIUM
		lowp vec4 v4TextureCube = texture(texCubeMap, v_v3Reflection);
		lowp float fSpec = u2.fSpecular * ((v4Texture.r + v4Texture.g + v4Texture.b) * 0.333);

		lowp vec3 v3Out = v4Texture.rgb + v4TextureCube.rgb * fSpec;
	#endif

	#ifdef DETAIL_LOW
		lowp vec3 v3Out = v4Texture.rgb;	
	#endif
	
	#ifdef REWIND
		v3Out = v3Out * (1.0 - u.fRewindEffect) + v4Texture.b * u.fRewindEffect;
	#endif
	
	#ifdef DETAIL_HIGH
		lowp vec3 v3Tmp = 1.0 - v3Out;
		out_v4Color.rgb = v3Out * (2.0 - u.fBrightness) + (1.0 - v3Tmp * v3Tmp * v3Tmp) * (u.fBrightness - 1.0);	
	#else
		out_v4Color.rgb = v3Out;
	#endif

	out_v4Color.a = v4Texture.a;
}
