
layout(binding = 0) uniform U
{    
	mat4 m44ModelViewProjection;
	mat3 m33SpecularRotation;
	mediump vec3 v3EyePosition;
	lowp float fBrightness;
	lowp float fRewindEffect;
	float fGamma;
	float fZOffset;
	float fZClamp;
	float fMinZOffset;  
} u;

layout(set = 2, binding = 0) uniform U2
{
	float fAlphaLevel;
	lowp float fSpecular;
} u2;

in vec4 a_v4Position;
in vec2 a_v2TexCoord;
in vec4 a_v4Color;

layout(location = 0) out vec2 v_v2TexCoord;
layout(location = 1) out vec4 v_v4Color;

#ifndef DETAIL_LOW
	in vec3 a_v3Normal;
#endif

#ifdef DETAIL_HIGH
	layout(location = 2) out lowp vec3 v_v3Normal;
	layout(location = 3) out mediump vec3 v_v3WorldPosition;
#endif

#ifdef DETAIL_MEDIUM
	layout(location = 4) out mediump vec3 v_v3Reflection;
#endif

void main() 
{
	v_v2TexCoord = a_v2TexCoord;
	v_v4Color = a_v4Color;
	v_v4Color.a *= u2.fAlphaLevel;	
	
	gl_Position = u.m44ModelViewProjection * a_v4Position;
	float fDiv = gl_Position.w;
	if (fDiv < u.fZClamp)
		fDiv = u.fZClamp;
	float zOffset = u.fZOffset / fDiv;
	if (zOffset < u.fMinZOffset)
		zOffset = u.fMinZOffset;
	gl_Position.z -= zOffset;

	#ifdef DETAIL_HIGH
		v_v3Normal = a_v3Normal * u.m33SpecularRotation;
		v_v3WorldPosition = a_v4Position.xyz * u.m33SpecularRotation;
	#endif

	#ifdef DETAIL_MEDIUM
		vec3 v3Normal = a_v3Normal * u.m33SpecularRotation;			
		mediump vec3 v3Incident = a_v4Position.xyz * u.m33SpecularRotation - u.v3EyePosition;
		mediump vec3 v3Reflection = v3Incident - 2.0 * dot(v3Incident, v3Normal) * v3Normal / dot(v3Normal, v3Normal);		
		v_v3Reflection = v3Reflection;	
	#endif
}


