precision lowp float;

uniform lowp samplerCube tex3;
uniform lowp sampler2D tex1;
uniform lowp sampler2D tex2;
uniform lowp sampler2D tex4;
uniform lowp sampler2D tex5;
uniform sampler2D tex7;

varying mediump vec2 v_v2TexCoord;
varying mediump vec3 v_v3WorldPosition;
varying mediump vec2 v_v2TexProjectionDeck;
varying mediump vec2 v_v2TexProjectionGrip;
varying mediump vec2 v_v2WearUV;

uniform lowp mat3 u_m33NormalTransform;
uniform lowp vec4 u_v4Color;
uniform lowp vec3 u_v3LightDir;
uniform mediump vec3 u_v3EyePosition;
uniform lowp vec3 u_v3WearColour;
uniform lowp float u_fMode;

void main() 
{	
	lowp vec3 v3TextureNormal = texture2D(tex1, v_v2TexCoord).rgb; 
	lowp vec3 v3TextureDiffuse = texture2D(tex2, v_v2TexCoord).rgb;
	lowp vec3 v3TextureWearSpecAo = texture2D(tex7, v_v2TexCoord).rgb; // r contains specular, g contains wear, b contains ambient oclusion
	
	lowp vec4 v4TexTemp;
	lowp vec4 v4TexTemp2;   	
	
	// Only apply custom image to grip and deck
	if (u_fMode < 0.5)
	{
		// Grip
		v4TexTemp = texture2D(tex5, v_v2TexProjectionGrip);
		v4TexTemp.a = v4TexTemp.a * 0.6;
		v3TextureDiffuse.rgb = (v3TextureDiffuse.rgb * (1.0 - v4TexTemp.a)) + (v3TextureDiffuse.rgb * v4TexTemp.rgb * v4TexTemp.a * 4.0);
		v3TextureDiffuse.rgb += v4TexTemp.a * v4TexTemp.rgb * 0.1;
		v3TextureWearSpecAo.b += v4TexTemp.a * v4TexTemp.r * 0.5;
	}
	else if (u_fMode < 1.5)
	{
		// Deck
		v4TexTemp = texture2D(tex5, v_v2TexProjectionDeck);
		v4TexTemp.a = v4TexTemp.a * 0.8;
		v3TextureDiffuse.rgb = (v3TextureDiffuse.rgb * (1.0 - v4TexTemp.a)) + (v3TextureDiffuse.rgb * v4TexTemp.rgb * v4TexTemp.a * 2.9);
	}	
	
	// Add wear for deck, grip and trucks
	mediump vec4 v4TextureWear = texture2D(tex4, v_v2WearUV);
	lowp vec4 v4Wear = vec4(u_v3WearColour, (v3TextureWearSpecAo.g - 1.0) / (0.001 + v4TextureWear.r) + 1.0);
	v4Wear.a = clamp(v4Wear.a, 0.0, 1.0);
	
	lowp float fA = clamp((1.0 - v4Wear.a * 1.5), 0.0, 1.5);

	v3TextureDiffuse.xyz = (v4Wear.rgb * v4Wear.a * v4Wear.a) + (v3TextureDiffuse.xyz * fA);
	v3TextureWearSpecAo.r *= fA;

	// todo: Is there a way to pre transform to avoid multiplying by u_m33NormalTransform during the fragment shader?
	// todo: I think it might require the cube map to be rotated, which wont work for different car body parts.
	// todo: It is an expensive per pixel operation so it would be nice to avoid it.
	mediump vec3 v3Incident = v_v3WorldPosition - u_v3EyePosition;
	lowp vec3 v3Normal = (2.0 * (v3TextureNormal.xyz - vec3(0.5, 0.5, 0.5))) * u_m33NormalTransform;
	v3Incident = reflect(v3Incident, v3Normal);
	
	lowp vec4 v4TextureCube = textureCube(tex3, v3Incident);
	
	gl_FragColor.xyz = 
		v4TextureCube.xyz * v3TextureWearSpecAo.r * v3TextureWearSpecAo.b * 1.5 + 
		v3TextureDiffuse.xyz * (dot(v3Normal, u_v3LightDir) + 1.5) * u_v4Color.xyz * v3TextureWearSpecAo.b;
				
	gl_FragColor.w = 0.25;
}
