//precision lowp float;

layout(location = 0) out vec4 out_v4Color;

layout(set = 0, binding = 1003) uniform samplerCube tex3;
layout(set = 1, binding = 1001) uniform sampler2D tex1;
layout(set = 1, binding = 1002) uniform sampler2D tex2;
layout(set = 1, binding = 1004) uniform sampler2D tex4;
#if defined (CUSTOMIZABLE_GRIP) || defined (CUSTOMIZABLE_DECK)
layout(set = 1, binding = 1005) uniform sampler2D tex5;
#endif
layout(set = 1, binding = 1007) uniform sampler2D tex7;

in vec2 v_v2TexCoord;
in vec3 v_v3WorldPosition;
in vec2 v_v2WearUV;

#if defined (CUSTOMIZABLE_GRIP) || defined (CUSTOMIZABLE_DECK)
in vec2 v_v2TexProjection;
#endif

layout(set = 0, binding = 0) uniform U0
{    
	uniform vec4 v4Color;
	uniform vec3 v3EyePosition;
	uniform float fBrightness;	
	uniform vec3 v3LightDir;
} u0;

layout(set = 1, binding = 2) uniform U2
{    
	uniform mat3 m33NormalTransform;	
	uniform vec3 v3WheelColour;
	uniform float fWearAmount;
	uniform vec3 v3WearColour;
} u2;

void main() 
{	
	vec3 v3TextureNormal = texture(tex1, v_v2TexCoord).rgb; 
	vec3 v3TextureDiffuse = texture(tex2, v_v2TexCoord).rgb;
	vec3 v3TextureWearSpecAo = texture(tex7, v_v2TexCoord).rgb; // r contains specular, g contains wear, b contains ambient oclusion
		
#ifdef CUSTOMIZABLE_GRIP
	// todo
	vec4 v4TexTemp = texture(tex5, v_v2TexProjection);
	v4TexTemp.a = v4TexTemp.a * 0.6;
	v3TextureDiffuse.rgb = (v3TextureDiffuse.rgb * (1.0 - v4TexTemp.a)) + (v3TextureDiffuse.rgb * v4TexTemp.rgb * v4TexTemp.a * 4.0);
	v3TextureDiffuse.rgb += v4TexTemp.a * v4TexTemp.rgb * 0.1;
	v3TextureWearSpecAo.b += v4TexTemp.a * v4TexTemp.r * 0.5;
#endif
#ifdef CUSTOMIZABLE_DECK
	vec4 v4TexTemp = texture(tex5, v_v2TexProjection);
	v4TexTemp.a = v4TexTemp.a * 0.8;
	v3TextureDiffuse.rgb = (v3TextureDiffuse.rgb * (1.0 - v4TexTemp.a)) + (v3TextureDiffuse.rgb * v4TexTemp.rgb * v4TexTemp.a * 2.9);
#endif
		
	// todo: Is there a way to pre transform to avoid multiplying by u.m33NormalTransform during the fragment shader?
	// todo: I think it might require the cube map to be rotated, which wont work for different car body parts.
	// todo: It is an expensive per pixel operation so it would be nice to avoid it.
	vec3 v3Incident = v_v3WorldPosition - u0.v3EyePosition;
	vec3 v3Normal = (2.0 * (v3TextureNormal.xyz - vec3(0.5, 0.5, 0.5))) * u2.m33NormalTransform;
	v3Incident = reflect(v3Incident, v3Normal);	

	vec4 v4TextureCube = texture(tex3, v3Incident);
	vec4 v4TextureWear = texture(tex4, v_v2WearUV);		
	 
	vec4 v4Wear = vec4(u2.v3WearColour, (v3TextureWearSpecAo.g - 1.0) / (0.001 + v4TextureWear.r) + 1.0);
	v4Wear.a = clamp(v4Wear.a, 0.0, 1.0);
	
	float fA = clamp((1.0 - v4Wear.a * 1.5), 0.0, 1.5);

	v3TextureDiffuse.xyz = (v4Wear.rgb * v4Wear.a * v4Wear.a) + (v3TextureDiffuse.xyz * fA);
	v3TextureWearSpecAo.r *= fA;

	out_v4Color.xyz = 
		v4TextureCube.xyz * v3TextureWearSpecAo.r * v3TextureWearSpecAo.b * 1.5 + 
		v3TextureDiffuse.xyz * (dot(v3Normal, u0.v3LightDir) + 1.5) * u0.v4Color.xyz * v3TextureWearSpecAo.b;
						
	vec3 v3Tmp = 1.0 - out_v4Color.rgb;

	out_v4Color.rgb = out_v4Color.rgb * (2.0 - u0.fBrightness) + (1.0 - v3Tmp * v3Tmp * v3Tmp) * (u0.fBrightness - 1.0);
		
	out_v4Color.w = 0.25;
}
