precision lowp float;

layout(location = 0) out vec4 out_v4Color;

layout(set = 1, binding = 1001) uniform lowp sampler2D tex;
#ifndef DETAIL_LOW
	layout(set = 0, binding = 1002) uniform lowp samplerCube texCubeMap;
#endif
#ifndef DETAIL_LOW
	layout(set = 1, binding = 1003) uniform lowp sampler2D texSpecular;
#endif
#ifdef SIGNAGE
	layout(set = 1, binding = 1004) uniform lowp sampler2D texSignage;
	layout(location = 4) in mediump vec2 v_v2TexCoordSignage;
	layout(location = 5) in lowp vec4 v_v4SignageModifier;
#endif

layout(location = 0) in mediump vec2 v_v2TexCoord;
#ifdef DETAIL_HIGH
	layout(location = 1) in lowp vec3 v_v3Normal;
	layout(location = 2) in mediump vec3 v_v3WorldPosition;
#elif defined (DETAIL_MEDIUM)
	layout(location = 3) in mediump vec3 v_v3Reflection;
#endif

layout(binding = 0) uniform U
{    
	mat4 m44ModelViewProjection;
	mat3 m33SpecularRotation;
	mediump vec3 v3EyePosition;
	lowp float fBrightness;
	lowp float fRewindEffect;
	float fGamma;
	float fZOffset;
	float fZClamp;
	float fMinZOffset;  
} u;

void main() 
{
	lowp vec4 v4Texture = texture(tex, v_v2TexCoord);
	#ifndef DETAIL_LOW
		lowp float fSpecular = texture(texSpecular, v_v2TexCoord).r;	
	#endif
	#ifdef SIGNAGE
		lowp vec4 v4TextureSignage = texture(texSignage, v_v2TexCoordSignage);
		lowp vec3 v3SignageMod = vec3(0.65, 0.25, 0.0);
		v4Texture = ((v4TextureSignage - v_v4SignageModifier.y) * v_v4SignageModifier.x) + v4Texture;
	#endif

	#ifdef DETAIL_HIGH
		mediump vec3 v3Incident = v_v3WorldPosition - u.v3EyePosition;
		mediump vec3 v3Reflection = v3Incident - 2.0 * dot(v3Incident, v_v3Normal) * v_v3Normal / dot(v_v3Normal, v_v3Normal);	
		lowp vec4 v4texture = texture(texCubeMap, v3Reflection);
		
		#ifdef REWIND
			lowp vec4 v4Tmp1 = (v4Texture + v4texture * fSpecular) * (1.0 - u.fRewindEffect) + v4Texture.b * u.fRewindEffect;	
		#else
			lowp vec4 v4Tmp1 = (v4Texture + v4texture * fSpecular);
		#endif		

		lowp vec4 v4Tmp2 = 1.0 - v4Tmp1;
		out_v4Color = v4Tmp1 * (2.0 - u.fBrightness) + (1.0 - v4Tmp2 * v4Tmp2 * v4Tmp2) * (u.fBrightness - 1.0);	

	#elif defined (DETAIL_MEDIUM)
		lowp vec4 v4texture = texture(texCubeMap, v_v3Reflection);
		#ifdef REWIND
			out_v4Color = (v4Texture + v4texture * fSpecular) * (1.0 - u.fRewindEffect) + v4Texture.b * u.fRewindEffect;	
		#else
			out_v4Color = v4Texture + v4texture * fSpecular;
		#endif	
	#else
		#ifdef REWIND
			out_v4Color = v4Texture * (1.0 - u.fRewindEffect) + v4Texture.b * u.fRewindEffect;
		#else
			out_v4Color = v4Texture;
		#endif
	#endif	

	//#ifdef SIGNAGE
	//		out_v4Color = v_v4SignageModifier;
	//#endif
	
}

