precision lowp float;

layout(location = 0) out vec4 out_v4Color;

#define REWIND
#define GAMMA

layout(set = 0, binding = 0) uniform U
{    
	mat4 m44ModelViewProjection;
	mat3 m33SpecularRotation;
	mediump vec3 v3EyePosition;
	lowp float fBrightness;
	lowp float fRewindEffect;
	float fGamma;
	float fZOffset;
	float fZClamp;
	float fMinZOffset;  
} u;

layout(set = 1, binding = 1) uniform U2
{
	lowp vec4 v4Color;
	float fAlphaLevel;
} u2;

#ifdef COMPOSIT_CHANNELS
layout(set = 1, binding = 2) uniform U3
{    
	lowp float fBlueBlendMode;
	lowp float fBlueIgnoreBaseColour;
	lowp vec3 v3BlueShadowColour;
	lowp vec3 v3BlueHighlightColour;	
	lowp float fBlueBlendMult;
	lowp float fBlueBlendOffset;
	lowp float fBlueSpecular;

	lowp float fGreenBlendMode;
	lowp float fGreenIgnoreBaseColour;
	lowp vec3 v3GreenShadowColour;
	lowp vec3 v3GreenHighlightColour;
	lowp float fGreenBlendMult;
	lowp float fGreenBlendOffset;
	lowp float fGreenSpecular;
} u3;
#endif

//#ifdef DECAL
//#endif

layout(set = 1, binding = 1001) uniform lowp sampler2D tex;
layout(set = 1, binding = 1004) uniform lowp sampler2D texLightmap;
in highp vec2 v_v2TexCoord;
in mediump vec2 v_v2TexCoordLightMap;

#ifdef DISTANCE_FADE
in mediump float v_fDistanceFade;
#endif



//--------------------------------------
#ifdef DETAIL_LOW

void main() 
{
	lowp vec4 v4Texture = texture(tex, v_v2TexCoord);
	lowp vec3 v3Light = texture(texLightmap, v_v2TexCoordLightMap).rgb;
	
	lowp vec3 v3Out = v4Texture.rgb * v3Light * u2.v4Color.rgb;

	#ifdef REWIND
		out_v4Color.rgb = v3Out * (1.0 - u.fRewindEffect) + v3Out.b * u.fRewindEffect;
	#else
		out_v4Color.rgb = v3Out;
	#endif

	
	#ifdef DECAL
		out_v4Color.a = v4Texture.a * u2.fAlphaLevel;
	#elif defined(TRANSPARENT)
    #ifdef DISTANCE_FADE
        float fDistanceFade = v_fDistanceFade;
        if (fDistanceFade > 1.0)
            fDistanceFade = 1.0;
        out_v4Color.a = v4Texture.a * fDistanceFade;
		#else
			out_v4Color.a = v4Texture.a;
		#endif
	#endif
}

#endif // #ifdef DETAIL_LOW

#ifndef DETAIL_LOW
//--------------------------------------

layout(set = 0, binding = 1002) uniform lowp samplerCube texCubeMap;
#ifdef DETAIL_HIGH
layout(set = 1, binding = 1003) uniform lowp sampler2D texSpecular;
in lowp vec3 v_v3Normal;
in mediump vec3 v_v3WorldPosition;
#endif

#ifdef DETAIL_MEDIUM
	in mediump vec3 v_v3Reflection;
#endif

#ifdef COMPOSIT_CHANNELS
	in lowp vec4 v_v4Color;
	in lowp vec4 v_v4Color2;


	#define MaxComponent(v3Value) max(max(v3Value.x, v3Value.y), v3Value.z)
//	lowp float MaxComponent(lowp vec3 v3Value)
//	{
//		return max(max(v3Value.x, v3Value.y), v3Value.z);
//	}

/* ---------------------------------------------------------------
	Functions seem to break old Android devices for some reason.  
	Leaving these here because I'd love to use them again but commented out and made inline for now
   ---------------------------------------------------------------
	void GreenMagicBlend(inout lowp vec3 v3Diffuse, inout lowp float fSpecular, lowp float fInterpolate, lowp float fTo, lowp vec3 v3ColourOveride)
	{
		lowp vec3 v3Blend1 = v3Diffuse * 1.0 / mix(MaxComponent(v3Diffuse), 1.0, u3.fGreenBlendMode);
	
		v3Blend1 = mix(v3Blend1, vec3(1.0, 1.0, 1.0), u3.fGreenIgnoreBaseColour);
		v3Blend1 *= mix(u3.v3GreenShadowColour, u3.v3GreenHighlightColour, fTo) * v3ColourOveride;

		fInterpolate = fInterpolate * u3.fGreenBlendMult - u3.fGreenBlendOffset * fTo; 
		fInterpolate = clamp(fInterpolate, 0.0, 1.0);

		v3Diffuse = mix(v3Diffuse, v3Blend1, fInterpolate);

		fSpecular = mix(fSpecular, u3.fGreenSpecular * dot(v3Blend1, vec3(1.0 / 3.3, 1.0 / 3.3, 1.0 / 3.3)), fInterpolate);
	}

	void BlueMagicBlend(inout lowp vec3 v3Diffuse, inout lowp float fSpecular, lowp float fInterpolate, lowp float fTo, lowp vec3 v3ColourOveride)
	{
		lowp vec3 v3Blend1 = v3Diffuse * 1.0 / mix(MaxComponent(v3Diffuse), 1.0, u3.fBlueBlendMode);
	
		v3Blend1 = mix(v3Blend1, vec3(1.0, 1.0, 1.0), u3.fBlueIgnoreBaseColour);
		v3Blend1 *= mix(u3.v3BlueShadowColour, u3.v3BlueHighlightColour, fTo) * v3ColourOveride;

		fInterpolate = fInterpolate * u3.fBlueBlendMult - u3.fBlueBlendOffset * fTo; 
		fInterpolate = clamp(fInterpolate, 0.0, 1.0);

		v3Diffuse = mix(v3Diffuse, v3Blend1, fInterpolate);
	
		fSpecular = mix(fSpecular,  u3.fBlueSpecular * dot(v3Blend1,vec3(1.0 / 3.3, 1.0 / 3.3, 1.0 / 3.3)), fInterpolate);		
	}

	float InvBlend(float fA, float fB) { return 1.0 - (1.0 - fA) * (1.0 - fB); }
	//float InvBlend(float fA, float fB) { return max(fA, fB); }

	// Alternate method that has more room for optimization by bringing things to the vertex shader and avoiding one per pixel device
	// However the end effect is a little different
	void MagicBlendGreenAndBlue(inout vec3 v3Diffuse, float fSpecular, float fInterpolateGreen, float fInterpolateBlue, float fToGreen, float fToBlue, vec3 v3ColourOveride)
	{
		float fMix = 0.0;
		if (fInterpolateBlue > 0.0)
			fMix = fInterpolateBlue * 1.05/ max(fInterpolateGreen, fInterpolateBlue);
		fMix = clamp(fMix, 0.0, 1.0);

		float fBlendMode = mix(u3.fGreenBlendMode, u3.fBlueBlendMode, fMix);
		float fIgnoreBaseColor = mix(u3.fGreenIgnoreBaseColour, u3.fBlueIgnoreBaseColour, fMix);
		vec3 v3ShadowColour = mix(u3.v3GreenShadowColour, u3.v3BlueShadowColour, fMix);
		vec3 v3HighlightColour = mix(u3.v3GreenHighlightColour, u3.v3BlueHighlightColour, fMix);
		float fSpecularMix = mix(u3.fGreenSpecular, u3.fBlueSpecular, fMix);

		vec3 v3Blend1 = v3Diffuse * 1.0 / mix(MaxComponent(v3Diffuse), 1.0, fBlendMode);
	
		v3Blend1 = mix(v3Blend1, vec3(1.0, 1.0, 1.0), fIgnoreBaseColor);
		v3Blend1 *= mix(v3ShadowColour, v3HighlightColour, mix(fToGreen, fToBlue, fMix)) * v3ColourOveride;

		float fInterpolate = InvBlend(
				clamp(fInterpolateGreen * u3.fGreenBlendMult - u3.fGreenBlendOffset * fToGreen, 0.0, 2.0) * (1.0 - fInterpolateBlue),
				clamp(fInterpolateBlue * u3.fBlueBlendMult - u3.fBlueBlendOffset * fToBlue, 0.0, 2.0) * fInterpolateBlue);

		fInterpolate = clamp(fInterpolate, 0.0, 1.0);

		v3Diffuse = mix(v3Diffuse, v3Blend1, fInterpolate);
	
		fSpecular = mix(fSpecular,  fSpecularMix * dot(v3Blend1,vec3(1.0 / 3.0, 1.0 / 3.0, 1.0 / 3.0)), fInterpolate);		
	}
*/	
#endif


void main()		
{
	lowp vec3 v3Out; 

	lowp vec4 v4Texture = texture(tex, v_v2TexCoord);
	lowp vec3 v3Diffuse = v4Texture.rgb;
	#ifdef DETAIL_HIGH	
		lowp vec3 v3MagicMap = texture(texSpecular, v_v2TexCoord).rgb;
		lowp float fSpecular = v3MagicMap.r;
	#endif
	lowp vec3 v3LightMap = texture(texLightmap, v_v2TexCoordLightMap).rgb;

	

	#ifdef COMPOSIT_CHANNELS
	// Making GreenMagicBlend() inline to fix old Android devices
//		GreenMagicBlend(v3Diffuse, fSpecular, v_v4Color.g, v3MagicMap.g, v_v4Color2.rgb);
	lowp float fInterpolate = v_v4Color.g;
	lowp float fTo = v3MagicMap.g;
	lowp vec3 v3ColourOveride = v_v4Color2.rgb;
//	void GreenMagicBlend(inout lowp vec3 v3Diffuse, inout lowp float fSpecular, lowp float fInterpolate, lowp float fTo, lowp vec3 v3ColourOveride)
//	{
		lowp vec3 v3Blend1 = v3Diffuse * 1.0 / mix(MaxComponent(v3Diffuse), 1.0, u3.fGreenBlendMode);
	
		v3Blend1 = mix(v3Blend1, vec3(1.0, 1.0, 1.0), u3.fGreenIgnoreBaseColour);
		v3Blend1 *= mix(u3.v3GreenShadowColour, u3.v3GreenHighlightColour, fTo) * v3ColourOveride;

		fInterpolate = fInterpolate * u3.fGreenBlendMult - u3.fGreenBlendOffset * fTo; 
		fInterpolate = clamp(fInterpolate, 0.0, 1.0);

		v3Diffuse = mix(v3Diffuse, v3Blend1, fInterpolate);

		fSpecular = mix(fSpecular, u3.fGreenSpecular * dot(v3Blend1, vec3(1.0 / 3.3, 1.0 / 3.3, 1.0 / 3.3)), fInterpolate);
//	}

	// Making BlueMagicBlend() inline to fix old Android devices
//		BlueMagicBlend(v3Diffuse, fSpecular, v_v4Color.b, v3MagicMap.b, v_v4Color2.rgb);
	fInterpolate = v_v4Color.b;
	fTo = v3MagicMap.b;
	v3ColourOveride = v_v4Color2.rgb;
//	void BlueMagicBlend(inout lowp vec3 v3Diffuse, inout lowp float fSpecular, lowp float fInterpolate, lowp float fTo, lowp vec3 v3ColourOveride)
//	{
		v3Blend1 = v3Diffuse * 1.0 / mix(MaxComponent(v3Diffuse), 1.0, u3.fBlueBlendMode);
	
		v3Blend1 = mix(v3Blend1, vec3(1.0, 1.0, 1.0), u3.fBlueIgnoreBaseColour);
		v3Blend1 *= mix(u3.v3BlueShadowColour, u3.v3BlueHighlightColour, fTo) * v3ColourOveride;

		fInterpolate = fInterpolate * u3.fBlueBlendMult - u3.fBlueBlendOffset * fTo; 
		fInterpolate = clamp(fInterpolate, 0.0, 1.0);

		v3Diffuse = mix(v3Diffuse, v3Blend1, fInterpolate);
	
		fSpecular = mix(fSpecular,  u3.fBlueSpecular * dot(v3Blend1,vec3(1.0 / 3.3, 1.0 / 3.3, 1.0 / 3.3)), fInterpolate);		
//	}
	#endif
	#ifdef DETAIL_HIGH	
		fSpecular *= u2.v4Color.a; 
	#endif

		
	v3Diffuse *= v3LightMap;
	v3Diffuse *= u2.v4Color.rgb;
	
	#ifdef DETAIL_HIGH	
		mediump vec3 v3Incident = v_v3WorldPosition - u.v3EyePosition;
		mediump vec3 v3Reflection = v3Incident - 2.0 * dot(v3Incident, v_v3Normal) * v_v3Normal / dot(v_v3Normal, v_v3Normal);	
	
		lowp vec3 v3texture = texture(texCubeMap, v3Reflection).rgb;
		
		v3Out = v3Diffuse + v3texture * fSpecular; 

		#ifdef ENABLE_FOG
			highp vec3 v3Incident2 = v_v3WorldPosition - u.v3EyePosition;

			// James: This line was causing crashes on older Android devices for some reason since the Vulkan update.
			//        Replacing length with a sqrt(dot()) should be functionally the same but doesn't crash devices.
//			v3Out += (1.0 - exp2(-0.0002 * length(v3Incident2))) * (vec3(0.7, 0.71, 0.72) - v3Out);
			float fIncidentLength = sqrt(dot(v3Incident2, v3Incident2));
			v3Out += (1.0 - exp2(-0.0002 * fIncidentLength)) * (vec3(0.7, 0.71, 0.72) - v3Out);
		#endif


	#endif

	#ifdef DETAIL_MEDIUM
		lowp vec4 v4texture = texture(texCubeMap, v_v3Reflection);
		lowp float fSpec =  (v4Texture.r + v4Texture.g + v4Texture.b);
		fSpec = u2.v4Color.a * fSpec * fSpec* 0.02;
		v3Out = v3Diffuse + v4texture.rgb * fSpec;
	#endif
	
		
	#ifdef GAMMA
		#ifdef DETAIL_HIGH	
			v3Out = pow(v3Out, vec3(u.fGamma));
		#else
			v3Out += (v3Out * v3Out - v3Out) * ((u.fGamma - 1.0) * 2.0);
		#endif
	#endif

	
	#ifdef REWIND
		out_v4Color.rgb = v3Out * (1.0 - u.fRewindEffect) + v3Out.b * u.fRewindEffect;
	#else
		out_v4Color.rgb = v3Out;
	#endif

	#ifdef DECAL
		out_v4Color.a = v4Texture.a * u2.fAlphaLevel;
	#elif defined(TRANSPARENT)
		#ifdef DISTANCE_FADE
            float fDistanceFade = v_fDistanceFade;
            if (fDistanceFade > 1.0)
                fDistanceFade = 1.0;
			out_v4Color.a = v4Texture.a * fDistanceFade;
		#else
			out_v4Color.a = v4Texture.a;
		#endif
	#endif
	
	//out_v4Color.rgb = v4Texture.rgb;
	//out_v4Color.rgb = v3LightMap;

//	out_v4Color.r = 0.0;
//	out_v4Color.g = 0.0;
//	out_v4Color.b = 0.0;
}

#endif //#ifndef DETAIL_LOW

