
layout(set = 0, binding = 0) uniform U
{    
	mat4 m44ModelViewProjection;
	mat3 m33SpecularRotation;
	mediump vec3 v3EyePosition;
	lowp float fBrightness;
	lowp float fRewindEffect;
	float fGamma;
	float fZOffset;
	float fZClamp;
	float fMinZOffset;  
} u;

#ifdef DIY
layout(set = 2, binding = 0) uniform U4
{
	mediump vec3 v3ObjectPosition;
	mediump vec3 v3CameraPosition;
	mat3 m33ObjectRotation;
} u4;
#endif

in vec4 a_v4Position;
in vec2 a_v2TexCoord;
in vec2 a_v2TexCoordLightMap;


out highp vec2 v_v2TexCoord;
out mediump vec2 v_v2TexCoordLightMap;

#ifdef DISTANCE_FADE
out mediump float v_fDistanceFade;
#endif


#ifndef DETAIL_LOW


	#ifdef DETAIL_HIGH
		out lowp vec3 v_v3Normal;
		out mediump vec3 v_v3WorldPosition;
	#endif

	#ifdef DETAIL_MEDIUM
		out mediump vec3 v_v3Reflection;
	#endif

	#ifdef COMPOSIT_CHANNELS
		in vec4 a_v4Color;
		in vec4 a_v4Color2;

		out lowp vec4 v_v4Color;
		out lowp vec4 v_v4Color2;
	#endif

	in vec3 a_v3Normal;

#endif //#ifndef DETAIL_LOW


#ifdef DISTANCE_FADE
in float a_fDistanceScaleFade;
#endif



void main() 
{
	v_v2TexCoord = a_v2TexCoord;
	v_v2TexCoordLightMap = a_v2TexCoordLightMap;
	

	gl_Position = u.m44ModelViewProjection * a_v4Position;

	#ifdef DISTANCE_FADE
		#ifdef DETAIL_HIGH
			v_fDistanceFade = 1.0;
		#else
			#ifdef DIY
				v_fDistanceFade = (5.0 - length(u4.v3CameraPosition - (u4.v3ObjectPosition + (u4.m33ObjectRotation * a_v4Position.xyz))) * 0.08 + a_fDistanceScaleFade * (5.0 * 0.08) );
			#else
				//v_fDistanceFade = (5.0 - length(gl_Position.xyz) * 0.08 + a_fDistanceScaleFade * (5.0 * 0.08) );
				v_fDistanceFade = (5.0 - length(u.v3EyePosition - a_v4Position.xyz) * 0.08 + a_fDistanceScaleFade * (5.0 * 0.08) );
			#endif
		#endif
	#endif
	
	#ifndef DETAIL_LOW

		#ifdef COMPOSIT_CHANNELS
			v_v4Color = a_v4Color;
			v_v4Color2 = a_v4Color2;
		#endif
		
		#ifdef DETAIL_MEDIUM
			vec3 v3Normal = a_v3Normal * u.m33SpecularRotation;			
			mediump vec3 v3Incident = a_v4Position.xyz * u.m33SpecularRotation - u.v3EyePosition;
			mediump vec3 v3Reflection = v3Incident - 2.0 * dot(v3Incident, v3Normal) * v3Normal / dot(v3Normal, v3Normal);		
			v_v3Reflection = v3Reflection;	
		#endif

		#ifdef DETAIL_HIGH
			v_v3Normal = a_v3Normal * u.m33SpecularRotation;
			v_v3WorldPosition = a_v4Position.xyz * u.m33SpecularRotation;
		#endif
	#endif //#ifndef DETAIL_LOW

	#ifdef Z_OFFSET
		float fDiv = gl_Position.w;
		if (fDiv < u.fZClamp)
			fDiv = u.fZClamp;
		float zOffset = u.fZOffset / fDiv;
		if (zOffset < u.fMinZOffset)
			zOffset = u.fMinZOffset;

		#ifndef DETAIL_LOW
			#ifdef Z_OFFSET_IGNORE_DOWNWARD_FACES
				float fNormalModifier = clamp((a_v3Normal.y + 0.1) * 10000.0, 0.0, 1.0);
				gl_Position.z -= (zOffset * fNormalModifier);
			#else
				gl_Position.z -= zOffset;
			#endif
		#else
			gl_Position.z -= zOffset;
		#endif
	#endif	
	
}

