precision lowp float;

uniform lowp vec4 u_v4Color;
uniform lowp sampler2D tex;
uniform lowp sampler2D texLightmap;
varying mediump vec2 v_v2TexCoord;
varying mediump vec2 v_v2LightTexCoord;
uniform lowp float u_fBrightness;
uniform mediump vec3 u_v3EyePosition;
#ifdef REWIND
uniform lowp float u_fRewindEffect;
#endif // #ifdef REWIND

//--------------------------------------
#ifdef DETAIL_LOW

varying mediump vec3 v_v3Reflection;

void main() 
{
	lowp vec3 v3Texture = texture2D(tex, v_v2TexCoord);
	lowp vec3 v3Light = texture2D(texLightmap, v_v2LightTexCoord);
	
	lowp vec3 v3Out = v3Texture * v3Light * u_v4Color.rgb;

	#ifdef REWIND
		gl_FragColor.rgb = v3Out * (1.0 - u_fRewindEffect) + v3Out.b * u_fRewindEffect;
	#else
		gl_FragColor.rgb = v3Out;
	#endif
}

#endif // #ifdef DETAIL_LOW

#ifndef DETAIL_LOW
//--------------------------------------

uniform lowp samplerCube texCubeMap;
uniform lowp sampler2D texSpecular;
varying lowp vec3 v_v3Normal;
varying mediump vec3 v_v3WorldPosition;
varying lowp vec4 v_v4Color;
varying lowp vec4 v_v4Color2;

uniform lowp float u_fBlueBlendMode;
uniform lowp float u_fBlueIgnoreBaseColour;
uniform lowp vec3 u_v3BlueShadowColour;
uniform lowp vec3 u_v3BlueHighlightColour;	
uniform lowp float u_fBlueBlendMult;
uniform lowp float u_fBlueBlendOffset;
uniform lowp float u_fBlueSpecular;

uniform lowp float u_fGreenBlendMode;
uniform lowp float u_fGreenIgnoreBaseColour;
uniform lowp vec3 u_v3GreenShadowColour;
uniform lowp vec3 u_v3GreenHighlightColour;
uniform lowp float u_fGreenBlendMult;
uniform lowp float u_fGreenBlendOffset;
uniform lowp float u_fGreenSpecular;

lowp float MaxComponent(lowp vec3 v3Value)
{
	return max(max(v3Value.x, v3Value.y), v3Value.z);
}

void GreenMagicBlend(inout lowp vec3 v3Diffuse, inout lowp float fSpecular, lowp float fInterpolate, lowp float fTo, lowp vec3 v3ColourOveride)
{
	lowp vec3 v3Blend1 = v3Diffuse * 1.0 / mix(MaxComponent(v3Diffuse), 1.0, u_fGreenBlendMode);
	
	v3Blend1 = mix(v3Blend1, vec3(1.0, 1.0, 1.0), u_fGreenIgnoreBaseColour);
	v3Blend1 *= mix(u_v3GreenShadowColour, u_v3GreenHighlightColour, fTo) * v3ColourOveride;

	fInterpolate = fInterpolate * u_fGreenBlendMult - u_fGreenBlendOffset * fTo; 
	fInterpolate = clamp(fInterpolate, 0.0, 1.0);

	v3Diffuse = mix(v3Diffuse, v3Blend1, fInterpolate);

	fSpecular = mix(fSpecular, u_fGreenSpecular * dot(v3Blend1, vec3(1.0 / 3.0, 1.0 / 3.0, 1.0 / 3.0)), fInterpolate);
}

void BlueMagicBlend(inout lowp vec3 v3Diffuse, inout lowp float fSpecular, lowp float fInterpolate, lowp float fTo, lowp vec3 v3ColourOveride)
{
	lowp vec3 v3Blend1 = v3Diffuse * 1.0 / mix(MaxComponent(v3Diffuse), 1.0, u_fBlueBlendMode);
	
	v3Blend1 = mix(v3Blend1, vec3(1.0, 1.0, 1.0), u_fBlueIgnoreBaseColour);
	v3Blend1 *= mix(u_v3BlueShadowColour, u_v3BlueHighlightColour, fTo) * v3ColourOveride;

	fInterpolate = fInterpolate * u_fBlueBlendMult - u_fBlueBlendOffset * fTo; 
	fInterpolate = clamp(fInterpolate, 0.0, 1.0);

	v3Diffuse = mix(v3Diffuse, v3Blend1, fInterpolate);
	
	fSpecular = mix(fSpecular,  u_fBlueSpecular * dot(v3Blend1,vec3(1.0 / 3.0, 1.0 / 3.0, 1.0 / 3.0)), fInterpolate);		
}

float InvBlend(float fA, float fB) { return 1.0 - (1.0 - fA) * (1.0 - fB); }
//float InvBlend(float fA, float fB) { return max(fA, fB); }

// Alternate method that has more room for optimization by bringing things to the vertex shader and avoiding one per pixel device
// However the end effect is a little different
void MagicBlendGreenAndBlue(inout vec3 v3Diffuse, float fSpecular, float fInterpolateGreen, float fInterpolateBlue, float fToGreen, float fToBlue, vec3 v3ColourOveride)
{
	float fMix = 0.0;
	if (fInterpolateBlue > 0.0)
		fMix = fInterpolateBlue * 1.05/ max(fInterpolateGreen, fInterpolateBlue);
	fMix = clamp(fMix, 0.0, 1.0);

	float fBlendMode = mix(u_fGreenBlendMode, u_fBlueBlendMode, fMix);
	float fIgnoreBaseColor = mix(u_fGreenIgnoreBaseColour, u_fBlueIgnoreBaseColour, fMix);
	vec3 v3ShadowColour = mix(u_v3GreenShadowColour, u_v3BlueShadowColour, fMix);
	vec3 v3HighlightColour = mix(u_v3GreenHighlightColour, u_v3BlueHighlightColour, fMix);
	float fSpecularMix = mix(u_fGreenSpecular, u_fBlueSpecular, fMix);

	vec3 v3Blend1 = v3Diffuse * 1.0 / mix(MaxComponent(v3Diffuse), 1.0, fBlendMode);
	
	v3Blend1 = mix(v3Blend1, vec3(1.0, 1.0, 1.0), fIgnoreBaseColor);
	v3Blend1 *= mix(v3ShadowColour, v3HighlightColour, mix(fToGreen, fToBlue, fMix)) * v3ColourOveride;

	float fInterpolate = InvBlend(
			clamp(fInterpolateGreen * u_fGreenBlendMult - u_fGreenBlendOffset * fToGreen, 0.0, 2.0) * (1.0 - fInterpolateBlue),
			clamp(fInterpolateBlue * u_fBlueBlendMult - u_fBlueBlendOffset * fToBlue, 0.0, 2.0) * fInterpolateBlue);

	fInterpolate = clamp(fInterpolate, 0.0, 1.0);

	v3Diffuse = mix(v3Diffuse, v3Blend1, fInterpolate);
	
	fSpecular = mix(fSpecular,  fSpecularMix * dot(v3Blend1,vec3(1.0 / 3.0, 1.0 / 3.0, 1.0 / 3.0)), fInterpolate);		
}


void main()		
{
	lowp vec3 v3Diffuse = texture2D(tex, v_v2TexCoord).rgb;
	lowp vec3 v3MagicMap = texture2D(texSpecular, v_v2TexCoord).rgb;
	lowp vec3 v3LightMap = texture2D(texLightmap, v_v2LightTexCoord).rgb;
	lowp float fSpecular = v3MagicMap.r;
	

	v3Diffuse *= u_v4Color.rgb;

	GreenMagicBlend(v3Diffuse, fSpecular, v_v4Color.g, v3MagicMap.g, v_v4Color2.rgb);
	BlueMagicBlend(v3Diffuse, fSpecular, v_v4Color.b, v3MagicMap.b, v_v4Color2.rgb);
	//MagicBlendGreenAndBlue(v3Diffuse, fSpecular, v_v4Color.g, v_v4Color.b, v3MagicMap.g, v3MagicMap.b, v_v4Color2.rgb);
	
	v3Diffuse *= v3LightMap;			
	fSpecular *= u_v4Color.a; 
	
	mediump vec3 v3Incident = v_v3WorldPosition - u_v3EyePosition;
	mediump vec3 v3Reflection = v3Incident - 2.0 * dot(v3Incident, v_v3Normal) * v_v3Normal / dot(v_v3Normal, v_v3Normal);	
	
	lowp vec4 v4TextureCube = textureCube(texCubeMap, v3Reflection);
	
	lowp vec3 v3Out = v3Diffuse + v4TextureCube.rgb * fSpecular;

	
	#ifdef REWIND
		gl_FragColor.rgb = v3Out * (1.0 - u_fRewindEffect) + v3Out.b * u_fRewindEffect;
	#else
		gl_FragColor.rgb = v3Out;
	#endif

}

#endif //#ifndef DETAIL_LOW


























































































