precision lowp float;

uniform lowp samplerCube tex3;
uniform lowp sampler2D tex1;
uniform lowp sampler2D tex2;
uniform lowp sampler2D tex4;
uniform lowp sampler2D tex5;
uniform lowp sampler2D tex7;

varying mediump vec2 v_v2TexCoord;
varying mediump vec3 v_v3WorldPosition;

uniform mediump mat3 u_m33NormalTransform;
uniform mediump vec4 u_v4Color;
uniform mediump vec3 u_v3LightDir;
uniform mediump vec3 u_v3EyePosition;
uniform mediump vec3 u_v3WheelColour;
uniform mediump vec3 u_v3WearColour;
uniform mediump float u_fWearAmount;

void main() 
{	
	lowp vec3 v3TextureNormal = texture2D(tex1, v_v2TexCoord).rgb; 
	lowp vec3 v3TextureDiffuse = texture2D(tex2, v_v2TexCoord).rgb;
	lowp vec3 v3TextureWearSpecAo = texture2D(tex7, v_v2TexCoord).rgb; // r contains specular, g contains wear, b contains ambient oclusion
	
	lowp float fRedAmmount = (v3TextureDiffuse.r - v3TextureDiffuse.g) * 1.9;
	v3TextureDiffuse.rgb = v3TextureDiffuse.r + fRedAmmount * (v3TextureDiffuse.r * u_v3WheelColour - v3TextureDiffuse.r);		

	lowp vec4 v4Wear = vec4(u_v3WearColour, (v3TextureWearSpecAo.g - 1.0) / (0.001 + u_fWearAmount) + 1.0);
	v4Wear.a = clamp(v4Wear.a, 0.0, 1.0);

	lowp float fA = clamp((1.0 - v4Wear.a * 1.5), 0.0, 1.5);

	v3TextureDiffuse.xyz = (v4Wear.rgb * v4Wear.a * v4Wear.a) + (v3TextureDiffuse.xyz * fA);
	v3TextureWearSpecAo.r *= fA;
	
	// todo: Is there a way to pre transform to avoid multiplying by u_m33NormalTransform during the fragment shader?
	// todo: I think it might require the cube map to be rotated, which wont work for different car body parts.
	// todo: It is an expensive per pixel operation so it would be nice to avoid it.
	mediump vec3 v3Incident = v_v3WorldPosition - u_v3EyePosition;
	lowp vec3 v3Normal = (2.0 * (v3TextureNormal.xyz - vec3(0.5, 0.5, 0.5))) * u_m33NormalTransform;
	v3Incident = reflect(v3Incident, v3Normal);
	
	lowp vec4 v4TextureCube = textureCube(tex3, v3Incident);
	
	gl_FragColor.xyz = 
		v4TextureCube.xyz * v3TextureWearSpecAo.r * v3TextureWearSpecAo.b * 1.5 + 
		v3TextureDiffuse.xyz * (dot(v3Normal, u_v3LightDir) + 1.5) * u_v4Color.xyz * v3TextureWearSpecAo.b;
				
	gl_FragColor.w = 0.25;
}
